/***************************************************************************
 *            client_dir_scan.c
 *
 *  Fri Oct 24 11:26:11 2003
 *  Copyright  2003  Phil Kett
 *  pkett@genesis-midi.com
 ****************************************************************************/
#include "../include/remsync.h"

int
client_dir_scan (int client_sd, char *src_dir, char *dest_dir, FILE * fd,
		 int port, char *servername)
{
	char message[BIG_BUF], command[BIG_BUF];
	char filename[BIG_BUF], csum[BIG_BUF];
	char fulldest[BIG_BUF], fullsrc[BIG_BUF], mode[BIG_BUF];
	char dummy1[BIG_BUF], dummy2[BIG_BUF];
	char owner[BIG_BUF], group[BIG_BUF];
	int dest_len;
	int src_len, filename_length, loop, uid, gid;
	int src_crc, dest_crc, need_file = 0, isdir, result, count;
	mode_t filemode;


	sprintf (command, "scan:%s\r\n", src_dir);
	send (client_sd, command, strlen (command), 0);

	umask (0000);
	while (1)
	{

		bzero (fulldest, BIG_BUF);
		bzero (fullsrc, BIG_BUF);
		bzero (filename, BIG_BUF);
		bzero (dummy1, BIG_BUF);
		bzero (dummy2, BIG_BUF);
		bzero (csum, BIG_BUF);
		bzero (owner, BIG_BUF);
		bzero (group, BIG_BUF);

		need_file = 0;
		fgets (message, BIG_BUF, fd);
		strip_string (message);
		if (strcmp (message, "DONE") == 0)
		{
			break;
		}
		if (conf.debug)
			printf ("client_dir_scan:Read %s\n", message);
		split_string (message, csum, dummy1);
		split_string (dummy1, fullsrc, dummy2);
		bzero (dummy1, BIG_BUF);
		split_string (dummy2, mode, dummy1);
		bzero (dummy2, BIG_BUF);
		split_string (dummy1, owner, dummy2);
		bzero (csum, BIG_BUF);
		split_string (dummy2, group, csum);
		if (conf.debug)
		{
			printf ("client_dir_scan: fullsrc: %s\n", fullsrc);
			printf ("client_dir_scan: mode: %s\n", mode);
			printf ("client_dir_scan: owner: %s\n", owner);
			printf ("client_dir_scan: group: %s\n", group);
			printf ("client_dir_scan: csum: %s\n", csum);
		}
		if (strcmp (fullsrc, src_dir) != 0)
		{
			dest_len = strlen (dest_dir);
			src_len = strlen (src_dir);
			filename_length = strlen (fullsrc);
			for (loop = src_len; loop < filename_length; loop++)
			{

				filename[loop - src_len] = fullsrc[loop];
			}
			filename[loop + 1] = '\0';
			strcat (fulldest, dest_dir);
			strcat (fulldest, filename);
		}
		else
		{
			strcat (fulldest, dest_dir);
		}
		strip_string (csum);
		filemode = atoi (mode);
		uid = atoi (owner);
		gid = atoi (group);
		if (strcmp (csum, "LINK") == 0)
		{
			/* this is a link -
			 * mode contains the target and not the permissions
			 */
			stats.num_links_processed += 1;
			result = symlink (mode, fulldest);
			if (result != 0)
			{
				if (conf.debug)
					logmsg ("Symlink!");
			}
		}
		else if (strcmp (csum, "DIRECTORY") == 0)
		{
			stats.num_dirs_processed += 1;
			if (conf.debug)
				printf ("******** DIRECTORY *******\n");
			/* This is a directory report
			 * make sure it exists
			 */
			isdir = is_a_dir (fulldest);
			if (isdir == -1)
			{
				stats.num_dirs_copied += 1;
				/* Directory doesn't exist - create it */
				if (conf.debug)
					printf ("Creating directory %s\n",
						fulldest);
				if (mkdir (fulldest, filemode) != 0)
				{
					logmsg ("Unable to create dir");
					exit (-1);
				}
				else
				{
					chown (fulldest, uid, gid);
				}
			}
		}
		else
		{
			if (conf.debug)
				printf ("Dest = %s\nSrc = %s\nFilename = %s\nCsum = %s\nSrc_dir = %s\ndest_dir = %s\n", fulldest, fullsrc, filename, csum, src_dir, dest_dir);
			/* check that filename exists */
			stats.num_files_processed += 1;
			if (file_exists (fulldest))
			{
				dest_crc = cksum (fulldest);
				src_crc = atoi (csum);

				if (dest_crc != src_crc)
				{
					if (conf.debug)
						printf ("File changed!\n");
					need_file = 1;
				}
			}
			else
			{
				need_file = 1;
			}
		}
		if (need_file == 1)
		{
			stats.num_files_copied += 1;
			if (conf.debug)
				syslog (LOG_INFO, "Getting file %s\n",
					fullsrc);
			if (strlen (conf.notify) != 0)
			{
				notify (fulldest);
			}
			for (count = 0; count < 5; count++)
			{	/* Attempt to copy the file over a maximum of 5 times */
				result = netgetfile (fullsrc, fulldest, port,
						     servername, filemode,
						     uid, gid);
				if (result == 0)
				{
					/* The file has copied over successfully, break the loop */
					break;
				}
				if (conf.debug)
					printf ("Retrying to get file %s\n",
						fullsrc);
			}
			/* We need the file so copy it over */
		}
	}
	return 1;
}
