/***************************************************************************
 *            remsync.c
 *
 *  Fri Oct 24 11:28:16 2003
 *  Copyright  2003  Phil Kett
 *  pkett@genesis-midi.com
 ****************************************************************************/

/*
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include "../include/remsync.h"

int
is_a_dir (char *filename)
{
	/*
	 * Check whether supplied filename is a directory
	 */
	struct stat Stat;
	int Tmp;
	if (conf.debug) printf("is_a_dir: checking %s\n",filename);
	Tmp = lstat (filename, &Stat);
	if (Tmp != 0)
	{
		if (conf.debug) printf ("Stat of %s Failed\n", filename);
		logmsg("Stat failed - is_a_dir\n");
		return -1;
	}
	if (S_ISDIR (Stat.st_mode))
	{
		if (conf.debug) printf("is_a_dir: Yes\n");
		return 1;
		
	}
	else
	{
		return 0;
	}
}
int
is_a_link (char *filename)
{
	/*
	 * Check whether supplied filename is a symlink
	 */
	struct stat Stat;
	int Tmp;
	if (conf.debug) printf("is_a_link: checking %s\n",filename);
	Tmp = lstat (filename, &Stat);
	if (Tmp != 0)
	{
		if (conf.debug) printf ("Stat of %s Failed\n", filename);
		logmsg("Stat failed - is_a_link\n");
		return -1;
	}
	if (S_ISLNK (Stat.st_mode))
	{
		if (conf.debug) printf("is_a_link: Yes\n");
		return 1;
	}
	else
	{
		return 0;
	}
}


void
dir_scan (char *basedir, char *destdir, int copy_all)
{
	char cursrc[BIG_BUF], curdest[BIG_BUF];
	struct dirent *d;
	DIR *dir;
	int srccrc, destcrc, isdir;


	dir = opendir (basedir);

	while ((d = readdir (dir)))
	{
		if (strcmp (d->d_name, ".") != 0
		    && strcmp (d->d_name, "..") != 0)
		{
			sprintf (curdest, "%s/%s", destdir, d->d_name);
			sprintf (cursrc, "%s/%s", basedir, d->d_name);
			if (conf.debug) printf("Checking %s\n",cursrc);
			isdir = is_a_dir (cursrc);
			if (isdir == 1)
			{
				if (conf.debug) printf ("Entering Directory %s\n", cursrc);
				if (copy_all == 0 && file_exists (curdest))
				{
					dir_scan (cursrc, curdest, 0);
				}
				else
				{
					mkdir (curdest,
					       (S_IRWXU | S_IRWXG | S_IRWXO));
					dir_scan (cursrc, curdest, 1);
				}
			}
			else if (isdir == 0)
			{
				if (copy_all == 1)
				{
					copy_file (cursrc, curdest);
				}
				else
				{
					if (file_exists (curdest))
					{
						if (conf.debug) printf ("%s exists, doing CRC\n", curdest);
						/*
						 * First check the size of the files, if the file sizes are different
						 * * then we don't need to do the CRC checking.
						 */
						if (size_check
						    (cursrc, curdest))
						{
							srccrc = cksum
								(cursrc);
							destcrc =
								cksum
								(curdest);
							if (conf.debug) printf ("CRCS: %u - %u\n", srccrc, destcrc);
							if (srccrc != destcrc)
							{
								copy_file
									(cursrc,
									 curdest);
							}
						}
						else
						{
							copy_file (cursrc,
								   curdest);
						}
					}
					else
					{
						copy_file (cursrc, curdest);
					}
				}
				if (conf.debug) printf("filename %s\n",cursrc);
			}
		}
	}
	closedir (dir);
}


int
file_exists (char *filename)
{
	struct stat file_stat;
	return ((stat (filename, &file_stat) == 0) ? 1 : 0);
}

void
getperms (char *filename, char *mode)
{
	struct stat file_stat;
	lstat (filename, &file_stat);
	sprintf(mode, "%d", file_stat.st_mode);
	//strmode (file_stat.st_mode, mode);
}
void getowner (char *filename, char *owner, char *group) 
{
	struct stat file_stat;
	lstat (filename, &file_stat);
	sprintf(owner, "%d", file_stat.st_uid);
	sprintf(group, "%d", file_stat.st_gid);
}

	
void checkslash (char *path) {
	int path_len;
	
	path_len = strlen(path);
	if (path[path_len] != '/') {
		if (conf.debug) printf("No trailing slash in %s\n", path);
		strcat(path, "/");
		if (conf.debug) printf("Result %s\n", path);
	}
	
}

#ifndef HAVE_STRMODE
void
strmode (mode_t mode, char *buffer)
{
	strcpy (buffer, "----------");
	if (S_ISDIR (mode))
	{
	buffer[0] = 'd';
	}
	else if (S_ISCHR (mode))
	{
	buffer[0] = 'c';
	}
	else if (S_ISBLK (mode))
	{
	buffer[0] = 'b';
	}
	else if (S_ISFIFO (mode))
	{
	buffer[0] = 'f';
	}
	else if (S_ISREG (mode))
	{
	buffer[0] = '?';
	}
	else if (S_ISLNK (mode))
	{
	buffer[0] = 'l';
	}
	else if (S_ISSOCK (mode))
	{
	buffer[0] = 's';
	}

	if (mode & S_IRUSR)
	{
	buffer[1] = 'r';
	}
	if (mode & S_IWUSR)
	{
	buffer[2] = 'w';
	}
	if (mode & S_IXUSR && mode & S_ISUID)
	{

	buffer[3] = 's';
	}
	else if (mode & S_ISUID)
	{
	buffer[3] = 'S';
	}
	else if (mode & S_IXUSR)
	{
	buffer[3] = 'x';
	}

	if (mode & S_IRGRP)
	{
	buffer[4] = 'r';
	}
	if (mode & S_IWGRP)
	{
	buffer[5] = 'w';
	}
	if (mode & S_IXGRP && mode & S_ISGID)
	{
	buffer[6] = 's';
	}
	else if (mode & S_ISGID)
	{
	buffer[6] = 'S';
	}
	else if (mode & S_IXGRP)
	{
	buffer[6] = 'x';
	}

	if (mode & S_IROTH)
	{
	buffer[7] = 'r';
	}
	if (mode & S_IWOTH)
	{
	buffer[8] = 'w';
	}
	if (mode & S_IXOTH && mode & S_ISVTX)
	{
	buffer[9] = 't';
	}
	else if (mode & S_ISVTX)
	{
		buffer[9] = 'T';
	}
	else if (mode & S_IXOTH)
	{
		buffer[9] = 'x';
	}
}
#endif

int dirdelete (char *directory) 
{
    struct dirent *d;
    DIR *dir;

    int isdir, result;

    char curdir[BIG_BUF];

	if (conf.debug) syslog(LOG_INFO, "Removing directory %s\n", directory);
    dir = opendir(directory);

    while ((d=readdir(dir))) {
        if (strcmp (d->d_name, ".") != 0 && strcmp(d->d_name, "..") != 0) {
            sprintf(curdir, "%s%s", directory, d->d_name);
            isdir = is_a_dir(curdir);
            if (isdir == 1) {
                checkslash(curdir);
                dirdelete(curdir);
            }
            else if (isdir == 0) {
                sprintf(curdir, "%s%s", directory, d->d_name);
				if (conf.debug) printf("Removing file %s\n", curdir);
                unlink(curdir);
            }
        }
    }
    closedir(dir);
    result = rmdir(directory);
    if (result != 0) {
		syslog(LOG_INFO, "Failed to delete directory %s\n", directory);
		return(-1);
	}
    return (0);
}
